@@ -119,7 +119,7 @@ function HistoryListModel(parent) {
         data.append("nzbfile", $(form.nzbFile)[0].files[0]);
         data.append("job", $('#modal-retry-job input[name="retry_job_id"]').val());
         data.append("password", $('#retry_job_password').val());
-        data.append("session", apiKey);
+        data.append("apikey", apiKey);
 
         // Add
         $.ajax({
@@ -169,7 +169,6 @@ function HistoryListModel(parent) {
 
     // Toggle showing failed
     self.toggleShowFailed = function(data, event) {
-
         // Set the loader so it doesn't flicker and then switch
         self.isLoading(true)
         self.showFailed(!self.showFailed())
@@ -177,7 +176,20 @@ function HistoryListModel(parent) {
         $('#history-options a').tooltip('hide')
         // Force refresh
         self.parent.refresh(true)
+    }
 
+    // Retry all failed
+    self.retryAllFailed = function(data, event) {
+        // Ask to be sure
+        if(confirm(glitterTranslate.retryAll)) {
+            // Send the command
+            callAPI({
+                mode: 'retry_all'
+            }).then(function() {
+                // Force refresh
+                self.parent.refresh(true)
+            })
+        }
     }
 
     // Empty history options
@@ -297,11 +309,11 @@ function HistoryModel(parent, data) {
     self.processingDownload = ko.pureComputed(function() {
         var status = self.status();
         // When we can cancel
-        if (status === 'Extracting' || status === 'Verifying' || status == 'Repairing') {
+        if (status === 'Extracting' || status === 'Verifying' || status == 'Repairing' || status === 'Running') {
             return 2
         }
         // These cannot be cancelled
-        if(status === 'Moving' ||  status === 'Running') {
+        if(status === 'Moving') {
             return 1
         }
         return false;
@@ -328,16 +340,14 @@ function HistoryModel(parent, data) {
             case 'speed':
                 // Anything to calculate?
                 if(self.historyStatus.bytes() > 0 && self.historyStatus.download_time() > 0) {
-                    var theSpeed = self.historyStatus.bytes()/self.historyStatus.download_time();
-                    theSpeed = theSpeed/1024;
-
-                    // MB/s or KB/s
-                    if(theSpeed > 1024) {
-                        theSpeed = theSpeed/1024;
-                        return theSpeed.toFixed(1) + ' MB/s'
-                    } else {
-                        return Math.round(theSpeed) + ' KB/s'
-                    }
+                    try {
+                        // Extract the Download section
+                        var downloadLog = ko.utils.arrayFirst(self.historyStatus.stage_log(), function(item) {
+                            return item.name() == 'Download'
+                        });
+                        // Extract the speed
+                        return downloadLog.actions()[0].match(/(\S*\s\S+)(?=<br\/>)/)[0]
+                    } catch(err) { }
                 }
                 return;
             case 'category':
@@ -356,10 +366,17 @@ function HistoryModel(parent, data) {
         return displayDateTime(self.completed(), parent.parent.dateFormat(), 'X')
     });
 
+    // Subscribe to retryEvent so we can load the password
+    self.canRetry.subscribe(function() {
+        self.updateAllHistory = true;
+    })
+
     // Re-try button
     self.retry = function() {
         // Set JOB-id
         $('#modal-retry-job input[name="retry_job_id"]').val(self.nzo_id)
+        // Set password
+        $('#retry_job_password').val(self.historyStatus.password())
         // Open modal
         $('#modal-retry-job').modal("show")
     };
