@@ -148,7 +148,6 @@ function QueueListModel(parent) {
         // See what the actual index is of the queue-object
         // This way we can see how we move up and down independent of pagination
         var itemReplaced = self.queueItems()[event.targetIndex+corTerm];
-
         callAPI({
             mode: "switch",
             value: itemMoved.id,
@@ -156,6 +155,25 @@ function QueueListModel(parent) {
         }).then(self.parent.refresh);
     };
 
+    // Move button clicked
+    self.moveButton = function(event,ui) {
+        var itemMoved = event;
+        var targetIndex;
+        if($(ui.currentTarget).is(".buttonMoveToTop")){
+            //we want to move to the top
+            targetIndex = 0;
+        } else {
+            // we want to move to the bottom
+			targetIndex = self.totalItems() - 1;
+        }
+        callAPI({
+            mode: "switch",
+            value: itemMoved.id,
+            value2: targetIndex
+        }).then(self.parent.refresh);
+
+    }
+
     // Save pagination state
     self.paginationLimit.subscribe(function(newValue) {
         // Save in config if global
@@ -171,7 +189,7 @@ function QueueListModel(parent) {
 
     // Do we show search box. So it doesn't dissapear when nothing is found
     self.hasQueueSearch = ko.pureComputed(function() {
-        return (self.pagination.hasPagination() || self.searchTerm())
+        return (self.pagination.hasPagination() || self.searchTerm() || (self.parent.hasQueue() && self.isMultiEditing()))
     })
 
     // Searching in queue (rate-limited in decleration)
@@ -453,6 +471,7 @@ function QueueListModel(parent) {
 function QueueModel(parent, data) {
     var self = this;
     self.parent = parent;
+    self.rawLabels = []
 
     // Job info
     self.id = data.nzo_id;
@@ -460,11 +479,15 @@ function QueueModel(parent, data) {
     self.password = ko.observable(data.password);
     self.index = ko.observable(data.index);
     self.status = ko.observable(data.status);
-    self.isGrabbing = ko.observable(data.status == 'Grabbing')
+    self.labels = ko.observableArray(data.labels);
+    self.isGrabbing = ko.observable(data.status == 'Grabbing' || data.avg_age == '-')
+    self.isFetchingBlocks = data.status == 'Fetching' || data.priority == 'Repair' // No need to update
     self.totalMB = ko.observable(parseFloat(data.mb));
-    self.remainingMB = ko.observable(parseFloat(data.mbleft));
+    self.remainingMB = ko.observable(parseFloat(data.mbleft))
+    self.missingMB = ko.observable(parseFloat(data.mbmissing))
+    self.percentage = ko.observable(parseInt(data.percentage))
     self.avg_age = ko.observable(data.avg_age)
-    self.missing = ko.observable(data.missing)
+    self.direct_unpack = ko.observable(data.direct_unpack)
     self.category = ko.observable(data.cat);
     self.priority = ko.observable(parent.priorityName[data.priority]);
     self.script = ko.observable(data.script);
@@ -476,8 +499,6 @@ function QueueModel(parent, data) {
     self.nameForEdit = ko.observable();
     self.editingName = ko.observable(false);
     self.hasDropdown = ko.observable(false);
-    self.rating_avg_video = ko.observable(false)
-    self.rating_avg_audio = ko.observable(false)
 
     // Color of the progress bar
     self.progressColor = ko.computed(function() {
@@ -485,8 +506,8 @@ function QueueModel(parent, data) {
         if(self.status() == 'Checking') {
             return '#58A9FA'
         }
-        // Check for missing data, the value is arbitrary!
-        if(self.missing() > 50) {
+        // Check for missing data, the value is arbitrary! (2%)
+        if(self.missingMB()/self.totalMB() > 0.02) {
             return '#F8A34E'
         }
         // Set to grey, only when not Force download
@@ -497,22 +518,23 @@ function QueueModel(parent, data) {
         return '';
     });
 
-    // MB's and percentages
-    self.downloadedMB = ko.computed(function() {
-        return(self.totalMB() - self.remainingMB()).toFixed(0);
-    });
-    self.percentageRounded = ko.pureComputed(function() {
-        return fixPercentages(((self.downloadedMB() / self.totalMB()) * 100).toFixed(2))
-    })
+    // MB's
     self.progressText = ko.pureComputed(function() {
-        return self.downloadedMB() + " MB / " + (self.totalMB() * 1).toFixed(0) + " MB";
+        return (self.totalMB() - self.remainingMB()).toFixed(0) + " MB / " + (self.totalMB() * 1).toFixed(0) + " MB";
     })
 
     // Texts
-    self.missingText= ko.pureComputed(function() {
-        // Check for missing data, the value is arbitrary!
-        if(self.missing() > 50) {
-            return self.missing() + ' ' + glitterTranslate.misingArt
+    self.name_title = ko.pureComputed(function() {
+        // When hovering over the job
+        if(self.direct_unpack()) {
+            return self.name() + ' - ' + glitterTranslate.status['DirectUnpack'] + ': ' + self.direct_unpack()
+        }
+        return self.name()
+    })
+    self.missingText = ko.pureComputed(function() {
+        // Check for missing data, the value is arbitrary! (1%)
+        if(self.missingMB()/self.totalMB() > 0.01) {
+            return self.missingMB().toFixed(0) + ' MB ' + glitterTranslate.misingArt
         }
         return;
     })
@@ -521,6 +543,10 @@ function QueueModel(parent, data) {
         if(self.status() == 'Checking') {
             return glitterTranslate.checking
         }
+        // Grabbing
+        if(self.status() == 'Grabbing') {
+            return glitterTranslate.fetch
+        }
         // Pausing status
         if((self.parent.parent.downloadsPaused() && self.priority() != 2) || self.pausedStatus()) {
             return glitterTranslate.paused;
@@ -529,6 +555,18 @@ function QueueModel(parent, data) {
         return rewriteTime(self.timeLeft());
     });
 
+    // Icon to better show force-priority
+    self.queueIcon = ko.computed(function() {
+        // Force comes first
+        if(self.priority() == 2) {
+            return 'glyphicon-forward'
+        }
+        if(self.pausedStatus()) {
+            return 'glyphicon-play'
+        }
+        return 'glyphicon-pause'
+    })
+
     // Extra queue column
     self.extraText = ko.pureComputed(function() {
         // Picked anything?
@@ -561,11 +599,13 @@ function QueueModel(parent, data) {
         self.password(data.password);
         self.index(data.index);
         self.status(data.status)
-        self.isGrabbing(data.status == 'Grabbing')
+        self.isGrabbing(data.status == 'Grabbing' || data.avg_age == '-')
         self.totalMB(parseFloat(data.mb));
         self.remainingMB(parseFloat(data.mbleft));
+        self.missingMB(parseFloat(data.mbmissing))
+        self.percentage(parseInt(data.percentage))
         self.avg_age(data.avg_age)
-        self.missing(data.missing)
+        self.direct_unpack(data.direct_unpack)
         self.category(data.cat);
         self.priority(parent.priorityName[data.priority]);
         self.script(data.script);
@@ -573,10 +613,12 @@ function QueueModel(parent, data) {
         self.pausedStatus(data.status == 'Paused');
         self.timeLeft(data.timeleft);
 
-        // If exists, otherwise false
-        if(data.rating_avg_video !== undefined) {
-            self.rating_avg_video(data.rating_avg_video === 0 ? '-' : data.rating_avg_video);
-            self.rating_avg_audio(data.rating_avg_audio === 0 ? '-' : data.rating_avg_audio);
+        // Did the label-list change?
+        // Otherwise KO will send updates to all texts during refresh()
+        if(self.rawLabels != data.labels.toString()) {
+            // Update
+            self.labels(data.labels);
+            self.rawLabels = data.labels.toString();
         }
     };
 
@@ -652,7 +694,7 @@ function QueueModel(parent, data) {
     }
     self.changeScript = function(item) {
         // Not on empty handlers
-        if(!item.script()) return;
+        if(!item.script() || parent.scriptsList().length <= 1) return;
         callAPI({
             mode: 'change_script',
             value: item.id,
@@ -668,7 +710,7 @@ function QueueModel(parent, data) {
     }
     self.changePriority = function(item, event) {
         // Not if we are fetching extra blocks for repair!
-        if(item.status() == 'Fetching') return
+        if(item.isFetchingBlocks) return
         callAPI({
             mode: 'queue',
             name: 'priority',
